/* ============================================================
 *    cl_info.c
 *       Written 2013 by Steffen Boerm, all rights reserved.
 *          ============================================================ */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <CL/cl.h>

int
main()
{
  cl_uint platforms, devices;
  cl_platform_id *platform;
  cl_device_id *device;
  cl_int result;
  cl_uint max_compute_units, max_clock_frequency;
  char *info;
  size_t infosize;
  cl_uint i, j;

  printf("========================================\n"
	 "Querying number of platforms\n");
  result = clGetPlatformIDs(0, 0, &platforms);
  assert(result == CL_SUCCESS);
  platform = (cl_platform_id *) malloc((size_t) sizeof(cl_platform_id) * platforms);
  result = clGetPlatformIDs(platforms, platform, 0);
  assert(result == CL_SUCCESS);
  printf("  %d platforms\n\n", platforms);

  for(i=0; i<platforms; i++) {
    printf("----------------------------------------\n"
	   "Querying platform %u\n", i);
    result = clGetPlatformInfo(platform[i], CL_PLATFORM_NAME, 0, 0, &infosize);
    info = (char *) malloc((size_t) sizeof(char) * infosize);
    result = clGetPlatformInfo(platform[i], CL_PLATFORM_NAME, infosize, info, 0);
    assert(result == CL_SUCCESS);
    printf("  Name: \"%s\"\n", info);
    free(info);
    result = clGetPlatformInfo(platform[i], CL_PLATFORM_VERSION, 0, 0, &infosize);
    assert(result == CL_SUCCESS);
    info = (char *) malloc((size_t) sizeof(char) * infosize);
    result = clGetPlatformInfo(platform[i], CL_PLATFORM_VERSION, infosize, info, 0);
    assert(result == CL_SUCCESS);
    printf("  OpenCL version: \"%s\"\n", info);
    free(info);
    result = clGetPlatformInfo(platform[i], CL_PLATFORM_PROFILE, 0, 0, &infosize);
    assert(result == CL_SUCCESS);
    info = (char *) malloc((size_t) sizeof(char) * infosize);
    result = clGetPlatformInfo(platform[i], CL_PLATFORM_PROFILE, infosize, info, 0);
    assert(result == CL_SUCCESS);
    printf("  OpenCL profile: \"%s\"\n", info);
    free(info);

    result = clGetDeviceIDs(platform[i], CL_DEVICE_TYPE_ALL, 0, 0, &devices);
    assert(result == CL_SUCCESS);
    device = (cl_device_id *) malloc((size_t) sizeof(cl_device_id) * devices);
    result = clGetDeviceIDs(platform[0], CL_DEVICE_TYPE_ALL, devices, device, 0);
    assert(result == CL_SUCCESS);
    printf("  %d devices\n", devices);
  
    for(j=0; j<devices; j++) {
      printf("  Querying device %u\n", j);
      result = clGetDeviceInfo(device[j], CL_DEVICE_NAME, 0, 0, &infosize);
      assert(result == CL_SUCCESS);
      info = (char *) malloc((size_t) sizeof(char) * infosize);
      result = clGetDeviceInfo(device[j], CL_DEVICE_NAME, infosize, info, 0);
      assert(result == CL_SUCCESS);
      printf("    Name: \"%s\"\n", info);
      free(info);
      
      result = clGetDeviceInfo(device[j], CL_DEVICE_MAX_COMPUTE_UNITS, sizeof(cl_uint), &max_compute_units, 0);
      assert(result == CL_SUCCESS);
      printf("    Compute units: %d\n", max_compute_units);
      
      result = clGetDeviceInfo(device[j], CL_DEVICE_MAX_CLOCK_FREQUENCY, sizeof(cl_uint), &max_clock_frequency, 0);
      assert(result == CL_SUCCESS);
      printf("    Frequency: %d\n", max_clock_frequency);
    }
    free(device);
  }
  free(platform);
  printf("========================================\n");

  return 0;
}

