
/* matrixmul_kernel.cl 
 * Matrix multiplication: C = A * B.
 * Device code.
 */
 
// OpenCL Kernel
__kernel void
matrixMul(__global float* C, 
          __global float* A, 
          __global float* B, 
          int wA, int wB)
{
  // Returns the unique global work-item ID value for dimension identified by dimindx. 
  //	The global work-item ID specifies the work-item ID based on the number of global work-items
  //	specified to execute the kernel. Valid values of dimindx are 0 to get_work_dim()  1. 
  //	For other values of dimindx, get_global_id() returns 0.
   int tx = get_global_id(0); 
   int ty = get_global_id(1);
 
   // value stores the element that is 
   // computed by the thread
   float value = 0;
   // Remember: #define WA 1024; Wa = WA at the host code [#define WB 1024]
   for (int k = 0; k < wA; ++k)
   {
      float elementA = A[ty * wA + k];
      float elementB = B[k * wB + tx];
      value += elementA * elementB;
   }
 
   // Write the matrix to device memory each 
   // thread writes one element
   C[ty * wA + tx] = value;
}
